package com.gemserk.animation4j.timeline;

import java.util.Comparator;
import java.util.LinkedList;

import com.gemserk.animation4j.interpolator.Interpolator;
import com.gemserk.animation4j.interpolator.NullInterpolator;

/**
 * Represents all the progress of a value inside a time line.
 * 
 * @author acoppes
 * 
 * @param <T>
 *            The type of the value.
 */
public class TimelineValue<T> {

	/**
	 * Determines a way to compare key frames based on the key frame time.
	 * 
	 * @author acoppes
	 */
	@SuppressWarnings("rawtypes")
	static class KeyFrameComparator implements Comparator<TimelineValue.KeyFrame> {
		@Override
		public int compare(TimelineValue.KeyFrame o1, TimelineValue.KeyFrame o2) {
			return (int) (o1.time - o2.time);
		}
	}

	/**
	 * A key frame specifies a specific value the variable should have in a specific time.
	 * 
	 * @param <T>
	 *            The type of the variable
	 * @author acoppes
	 */
	static class KeyFrame<T> {

		private float time;

		private T value;

		private Interpolator<T> interpolator;

		public float getTime() {
			return time;
		}

		public T getValue() {
			return value;
		}

		public Interpolator<T> getInterpolator() {
			return interpolator;
		}

		public KeyFrame(float time, T value, Interpolator<T> interpolator) {
			this.time = time;
			this.value = value;
			this.interpolator = interpolator;
		}

	}

	LinkedList<KeyFrame<T>> keyFrames = new LinkedList<KeyFrame<T>>();

	String name;

	public void setName(String name) {
		this.name = name;
	}

	public String getName() {
		return name;
	}
	
	TimelineValue() {
		
	}

	public void addKeyFrame(float time, T value, Interpolator<T> interpolator) {
		keyFrames.add(new KeyFrame<T>(time, value, interpolator));
	}

	public void addKeyFrame(float time, T value) {
		addKeyFrame(time, value, new NullInterpolator<T>());
	}

	/**
	 * Returns the value for the given time, interpolating between corresponding key frames.
	 * 
	 * @param time
	 *            The time to use when calculating the value.
	 * @return An interpolated value based on corresponding to specified time key frames.
	 */
	public T getValue(float time) {

		KeyFrame<T> firstKeyFrame = keyFrames.getFirst();

		if (time <= firstKeyFrame.time) {
			if (keyFrames.size() == 1)
				return firstKeyFrame.getValue();

			KeyFrame<T> secondKeyFrame = keyFrames.get(1);

			Interpolator<T> interpolator = firstKeyFrame.getInterpolator();
			return interpolator.interpolate(firstKeyFrame.getValue(), secondKeyFrame.getValue(), 0f);
		}

		for (int i = 0; i < keyFrames.size(); i++) {

			TimelineValue.KeyFrame<T> currentKeyFrame = keyFrames.get(i);

			if (currentKeyFrame.time > time) {

				TimelineValue.KeyFrame<T> previousKeyFrame = keyFrames.get(i - 1);

				if (previousKeyFrame == null)
					throw new RuntimeException("invalid time " + time);

				float interval = currentKeyFrame.getTime() - previousKeyFrame.getTime();

				float weight = (time - previousKeyFrame.getTime()) / interval;

				Interpolator<T> interpolator = previousKeyFrame.getInterpolator();
				return interpolator.interpolate(previousKeyFrame.getValue(), currentKeyFrame.getValue(), weight);

			}

		}

		if (keyFrames.size() == 1)
			return keyFrames.getLast().getValue();

		KeyFrame<T> secondLastFrame = keyFrames.get(keyFrames.size() - 2);
		KeyFrame<T> lastFrame = keyFrames.getLast();

		Interpolator<T> interpolator = secondLastFrame.getInterpolator();

		return interpolator.interpolate(secondLastFrame.getValue(), lastFrame.getValue(), 1f);
	}

}